@@ -78,10 +78,8 @@
     (license . ,(package-license package))
     (home-page . ,(package-home-page package))
     (maintainers . ("bug-guix@gnu.org"))
-
-    ;; Work around versions of 'hydra-eval-guile-jobs' before Hydra commit
-    ;; 61448ca (27 Feb. 2014) which used a default timeout of 2h.
-    (timeout . 72000)))
+    (timeout . ,(or (assoc-ref (package-properties package) 'timeout)
+                    72000)))) ; 20 hours by default
 
 (define (package-job store job-name package system)
   "Return a job called JOB-NAME that builds PACKAGE on SYSTEM."
@@ -103,7 +101,7 @@ SYSTEM."
   ;; Note: Don't put the '-final' package variants because (1) that's
   ;; implicit, and (2) they cannot be cross-built (due to the explicit input
   ;; chain.)
-  (list gcc-4.8 gcc-4.7 glibc binutils
+  (list gcc-4.8 gcc-4.9 gcc-5 glibc binutils
         gmp mpfr mpc coreutils findutils diffutils patch sed grep
         gawk gnu-gettext hello guile-2.0 zlib gzip xz
         %bootstrap-binaries-tarball
@@ -159,7 +157,7 @@ system.")
                        (set-guile-for-build (default-guile))
                        (system-disk-image installation-os
                                           #:disk-image-size
-                                          (* 850 MiB))))))
+                                          (* 860 MiB))))))
       '()))
 
 (define (tarball-jobs store system)
