@@ -853,10 +853,13 @@ OUTPUTS is a list of package outputs (may be an empty list)."
     (unless (and (null? install) (null? remove))
       (with-store store
         (let* ((derivation (run-with-store store
-                             (profile-derivation new-manifest)))
+                             (mbegin %store-monad
+                               (set-guile-for-build (default-guile))
+                               (profile-derivation new-manifest))))
                (derivations (list derivation))
                (new-profile (derivation->output-path derivation)))
           (set-build-options store
+                             #:print-build-trace #f
                              #:use-substitutes? use-substitutes?)
           (show-manifest-transaction store manifest transaction
                                      #:dry-run? dry-run?)
@@ -875,7 +878,8 @@ OUTPUTS is a list of package outputs (may be an empty list)."
                      (format #t (N_ "~a package in profile~%"
                                     "~a packages in profile~%"
                                     count)
-                             count))))))))))
+                             count)
+                     (display-search-paths entries profile))))))))))
 
 (define (delete-generations* profile generations)
   "Delete GENERATIONS from PROFILE.
@@ -883,6 +887,12 @@ GENERATIONS is a list of generation numbers."
   (with-store store
     (delete-generations store profile generations)))
 
+(define (package-location-string package-id)
+  "Return a location string of a package PACKAGE-ID."
+  (and-let* ((package  (package-by-id package-id))
+             (location (package-location package)))
+    (location->string location)))
+
 (define (package-source-derivation->store-path derivation)
   "Return a store path of the package source DERIVATION."
   (match (derivation-outputs derivation)
@@ -908,6 +918,7 @@ GENERATIONS is a list of generation numbers."
       (let* ((derivation  (package-source-derivation store source))
              (derivations (list derivation)))
         (set-build-options store
+                           #:print-build-trace #f
                            #:use-substitutes? use-substitutes?)
         (show-what-to-build store derivations
                             #:use-substitutes? use-substitutes?
