@@ -27,7 +27,7 @@
 (define-public calcurse
   (package
     (name "calcurse")
-    (version "3.1.4")
+    (version "3.2.1")
     (source
      (origin
       (method url-fetch)
@@ -35,15 +35,15 @@
                           version ".tar.gz"))
       (sha256
        (base32
-        "1qwhffwhfg7bjxrviwlcrhnfw0976d39da8kfspq6dgd9nqv68a1"))))
+        "0g92c4x02iv2wi3c1sm3nz1ay96bi2m7klxq3qyyx3pr11l53m85"))))
     (build-system gnu-build-system)
     (inputs `(("ncurses" ,ncurses)))
     (home-page "http://www.calcurse.org")
     (synopsis "Text-based calendar and scheduling")
     (description
-     "Calcurse is a text-based calendar and scheduling application. It helps
-keep track of events, appointments and everyday tasks. A configurable
+     "Calcurse is a text-based calendar and scheduling application.  It helps
+keep track of events, appointments and everyday tasks.  A configurable
 notification system reminds user of upcoming deadlines, and the curses based
-interface can be customized to suit user needs. All of the commands are
+interface can be customized to suit user needs.  All of the commands are
 documented within an online help system.")
     (license bsd-2)))
