@@ -107,5 +107,5 @@ Algorithms currently supplied are:
 CRC-32, CRC-32B, ALDER-32, MD-2, MD-4, MD-5, RIPEMD-128, RIPEMD-160,
 RIPEMD-256, RIPEMD-320, SHA-1, SHA-224, SHA-256, SHA-384, SHA-512, HAVAL-128,
 HAVAL-160, HAVAL-192, HAVAL-256, TIGER, TIGER-128, TIGER-160, GOST, WHIRLPOOL,
-SNEFRU-128, SNEFRU-256")
+SNEFRU-128, SNEFRU-256.")
     (license gpl2+)))
