@@ -1,5 +1,6 @@
 ;;; GNU Guix --- Functional package management for GNU
-;;; Copyright © 2013 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2013, 2015 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2015 Mark H Weaver <mhw@netris.org>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -94,17 +95,17 @@ anywhere.")
 (define-public samba
   (package
     (name "samba")
-    (version "3.6.8")
+    (version "3.6.25")
     (source (origin
              (method url-fetch)
-             (uri (string-append "http://us3.samba.org/samba/ftp/stable/samba-"
+             (uri (string-append "https://www.samba.org/samba/ftp/stable/samba-"
                                  version ".tar.gz"))
              (sha256
               (base32
-               "1phl6mmrc72jyvbyrw6cv6b92cxq3v2pbn1fh97nnb4hild1fnjg"))))
+               "0l9pz2m67vf398q3c2dwn8jwdxsjb20igncf4byhv6yq5dzqlb4g"))))
     (build-system gnu-build-system)
     (arguments
-     '(#:phases (alist-cons-before
+     `(#:phases (alist-cons-before
                  'configure 'chdir
                  (lambda _
                    (chdir "source3"))
@@ -113,19 +114,20 @@ anywhere.")
                   (lambda* (#:key outputs #:allow-other-keys)
                     (let* ((out (assoc-ref outputs "out"))
                            (lib (string-append out "/lib")))
-                      ;; Add LIB to the RUNPATH of all the executables.
+                      ;; Add LIB to the RUNPATH of all the executables and
+                      ;; dynamic libraries.
                       (with-directory-excursion out
                         (for-each (cut augment-rpath <> lib)
                                   (append (find-files "bin" ".*")
-                                          (find-files "sbin" ".*"))))))
+                                          (find-files "sbin" ".*")
+                                          (find-files "lib" ".*"))))))
                   %standard-phases))
 
        #:modules ((guix build gnu-build-system)
                   (guix build utils)
                   (guix build rpath)
                   (srfi srfi-26))
-       #:imported-modules ((guix build gnu-build-system)
-                           (guix build utils)
+       #:imported-modules (,@%gnu-build-system-modules
                            (guix build rpath))
 
        ;; This flag is required to allow for "make test".
@@ -166,14 +168,14 @@ Desktops into Active Directory environments using the winbind daemon.")
 (define-public talloc
   (package
     (name "talloc")
-    (version "2.1.0")
+    (version "2.1.2")
     (source (origin
               (method url-fetch)
-              (uri (string-append "http://www.samba.org/ftp/talloc/talloc-"
+              (uri (string-append "https://www.samba.org/ftp/talloc/talloc-"
                                   version ".tar.gz"))
               (sha256
                (base32
-                "13zh628hzlp2v9vj70knnfac2xbxqrdhgap30csq4zv4h8w3j087"))))
+                "13c365f7y8idjf2v1jxdjpkc3lxdmsxxfxjx1ymianm7zjiph393"))))
     (build-system gnu-build-system)
     (arguments
      '(#:phases (alist-replace
