@@ -101,7 +101,7 @@
   (let* ((file  (search-path %load-path "guix.scm"))
          (local (local-file file))
          (exp   (gexp (display (ungexp local))))
-         (intd  (add-to-store %store (basename file) #t
+         (intd  (add-to-store %store (basename file) #f
                               "sha256" file)))
     (and (gexp? exp)
          (match (gexp-inputs exp)
@@ -109,6 +109,35 @@
             (eq? x local)))
          (equal? `(display ,intd) (gexp->sexp* exp)))))
 
+(test-assert "one local file, symlink"
+  (let ((file (search-path %load-path "guix.scm"))
+        (link (tmpnam)))
+    (dynamic-wind
+      (const #t)
+      (lambda ()
+        (symlink (canonicalize-path file) link)
+        (let* ((local (local-file link "my-file" #:recursive? #f))
+               (exp   (gexp (display (ungexp local))))
+               (intd  (add-to-store %store "my-file" #f
+                                    "sha256" file)))
+          (and (gexp? exp)
+               (match (gexp-inputs exp)
+                 (((x "out"))
+                  (eq? x local)))
+               (equal? `(display ,intd) (gexp->sexp* exp)))))
+      (lambda ()
+        (false-if-exception (delete-file link))))))
+
+(test-assert "one plain file"
+  (let* ((file     (plain-file "hi" "Hello, world!"))
+         (exp      (gexp (display (ungexp file))))
+         (expected (add-text-to-store %store "hi" "Hello, world!")))
+    (and (gexp? exp)
+         (match (gexp-inputs exp)
+           (((x "out"))
+            (eq? x file)))
+         (equal? `(display ,expected) (gexp->sexp* exp)))))
+
 (test-assert "same input twice"
   (let ((exp (gexp (begin
                      (display (ungexp coreutils))
@@ -350,7 +379,7 @@
 
 (test-assertm "gexp->derivation, local-file"
   (mlet* %store-monad ((file ->  (search-path %load-path "guix.scm"))
-                       (intd     (interned-file file))
+                       (intd     (interned-file file #:recursive? #f))
                        (local -> (local-file file))
                        (exp ->   (gexp (begin
                                          (stat (ungexp local))
@@ -482,7 +511,7 @@
 
 (test-assertm "gexp->derivation #:references-graphs"
   (mlet* %store-monad
-      ((one (text-file "one" "hello, world"))
+      ((one (text-file "one" (random-text)))
        (two (gexp->derivation "two"
                               #~(symlink #$one #$output:chbouib)))
        (drv (gexp->derivation "ref-graphs"
@@ -507,15 +536,22 @@
                                           (guix build utils))))
        (ok? (built-derivations (list drv)))
        (guile-drv  (package->derivation %bootstrap-guile))
+       (bash       (interned-file (search-bootstrap-binary "bash"
+                                                           (%current-system))
+                                  "bash" #:recursive? #t))
        (g-one   -> (derivation->output-path drv "one"))
        (g-two   -> (derivation->output-path drv "two"))
        (g-guile -> (derivation->output-path drv)))
     (return (and ok?
                  (equal? (call-with-input-file g-one read) (list one))
-                 (equal? (call-with-input-file g-two read)
-                         (list one (derivation->output-path two "chbouib")))
-                 (equal? (call-with-input-file g-guile read)
-                         (list (derivation->output-path guile-drv)))))))
+                 (lset= string=?
+                        (call-with-input-file g-two read)
+                        (list one (derivation->output-path two "chbouib")))
+
+                 ;; Note: %BOOTSTRAP-GUILE depends on the bootstrap Bash.
+                 (lset= string=?
+                        (call-with-input-file g-guile read)
+                        (list (derivation->output-path guile-drv) bash))))))
 
 (test-assertm "gexp->derivation #:allowed-references"
   (mlet %store-monad ((drv (gexp->derivation "allowed-refs"
@@ -607,6 +643,17 @@
                                             file)))))
       #:guile-for-build (package-derivation %store %bootstrap-guile))))
 
+(test-assert "gexp->derivation vs. %current-target-system"
+  (let ((mval (gexp->derivation "foo"
+                                #~(begin
+                                    (mkdir #$output)
+                                    (foo #+gnu-make))
+                                #:target #f)))
+    ;; The value of %CURRENT-TARGET-SYSTEM at bind-time should have no
+    ;; influence.
+    (parameterize ((%current-target-system "fooooo"))
+      (derivation? (run-with-store %store mval)))))
+
 (test-assert "printer"
   (string-match "^#<gexp \\(string-append .*#<package coreutils.*\
  \"/bin/uname\"\\) [[:xdigit:]]+>$"
